﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using USB;
using System.Diagnostics;

namespace SignalGen
{
    public partial class frmSignalGen : Form
    {
        static USBWrapper uw;

        public frmSignalGen()
        {
            InitializeComponent();
        }

        private class Packet
        {
            public enum eType
            {
                GET_VERSION = 0x00,

                INITWRITE = 0x03,
                WRITERAM = 0x04,
                STARTGEN = 0x05,
                MANCLK = 0x06,
                FPGAHARDRESET = 0x07,

                SETDAC = 0x08,

                RESET = 0xFF
            };

            public eType Type;
            public byte[] Data;

            public byte[] GetData()
            {
                byte[] buf = new byte[Data.Length + 2];
                buf[0] = (byte)Type;
                buf[1] = (byte)Data.Length;
                Array.Copy(Data, 0, buf, 2, Data.Length);
                return buf;
            }

            public void SetData(byte[] buf)
            {
                Type = (eType)buf[0];
                Data = new byte[buf[1]];
                Array.Copy(buf, 2, Data, 0, Data.Length);
            }
        }

        private static Packet Execute(Packet packet)
        {
            byte[] buf = packet.GetData();
            byte[] rcvbuf = new byte[64];
            Packet rcv = new Packet();

            short sent = uw.Write(buf, 100);
            if (sent != buf.Length)
            {
                Console.WriteLine("Send Error");
                throw new ApplicationException("Packet Send Error");
            }

            short rec = uw.Read(ref rcvbuf, 100);
            rcv.SetData(rcvbuf);
            if (rec != rcv.Data.Length + 2)
            {
                Console.WriteLine("Receive Error");
                throw new ApplicationException("Packet Receive Error");
            }

            return rcv;
        }

        private void SignalGen_Load(object sender, EventArgs e)
        {
            uw = new USBWrapper();
            short sRet = uw.Open(0, "vid_04d8&pid_000c", "\\MCHP_EP1", "\\MCHP_EP1", false);

            if (sRet != 0)
            {
                Console.WriteLine("Could not open USB Device!");
                throw new ApplicationException("Could not open USB Device");
            }

            rbSine.Checked = true;
        }

        private void SignalGen_FormClosing(object sender, FormClosingEventArgs e)
        {
            uw.Close();
        }


        private void btnInitWrite_Click(object sender, EventArgs e)
        {
            Packet p = new Packet();
            Packet r;

            p.Type = Packet.eType.INITWRITE;
            p.Data = new byte[2];
            p.Data[0] = p.Data[1] = 0xff;

            Debug.WriteLine("Sent InitCount: " + BitConverter.ToString(p.GetData()).Replace('-', ' '));
            r = Execute(p);
            Debug.WriteLine("Received : " + BitConverter.ToString(r.GetData()).Replace('-', ' '));

        }

        private void btnWriteRam_Click(object sender, EventArgs e)
        {
            byte[] bsin = new byte[2048];
            FpgaHardReset();
            FpgaInitWrite();

            if (rbSine.Checked == true)
            {
                for (int i = 0; i < 2048; i++)
                {
                    bsin[i] = (byte)(Math.Sin(Math.PI * ((360.0 / 2048.0) * (double)i) / 180.0) * 127 + 128);
                }
            }

            if (rbSquare.Checked == true)
            {
                for (int i = 0; i < 2048; i++)
                {
                    bsin[i] = (byte)(i < 1024 ? 0x05 : 0xfa);
                }
            }

            if (rbSaw.Checked == true)
            {
                for (int i = 0; i < 2048; i++)
                {
                    bsin[i] = (byte)(i * (256.0 / 2048.0));
                }
            }

            for (int i = 0; i < 2048; i += 32)
            {
                WriteRam32(bsin, i);
            }
        }

        private void WriteRam32(byte[] data, int start)
        {
            Packet p = new Packet();
            Packet r;

            p.Type = Packet.eType.WRITERAM;
            p.Data = new byte[32];
            Array.Copy(data, start, p.Data, 0, 32);

            Debug.WriteLine("Sent WriteRam: " + BitConverter.ToString(p.GetData()).Replace('-', ' '));
            r = Execute(p);
            Debug.WriteLine("Received : " + BitConverter.ToString(r.GetData()).Replace('-', ' '));
        }

        private void btnClk_Click(object sender, EventArgs e)
        {
            Packet p = new Packet();
            Packet r;

            p.Type = Packet.eType.MANCLK;
            p.Data = new byte[1];            

            Debug.WriteLine("Sent Clock: " + BitConverter.ToString(p.GetData()).Replace('-', ' '));
            r = Execute(p);
            Debug.WriteLine("Received : " + BitConverter.ToString(r.GetData()).Replace('-', ' '));

        }

        private void btnStartGen_Click(object sender, EventArgs e)
        {
            Packet p = new Packet();
            Packet r;

            p.Type = Packet.eType.STARTGEN;
            p.Data = new byte[7];
            p.Data[0] = 0x01;//0x01;
            p.Data[1] = 0x40;
            p.Data[2] = 0x10;
            p.Data[3] = 0x05;

            p.Data[4] = 0x00;
            p.Data[5] = 0x00;

            p.Data[6] = 0x00;

            Debug.WriteLine("Sent Clock: " + BitConverter.ToString(p.GetData()).Replace('-', ' '));
            r = Execute(p);
            Debug.WriteLine("Received : " + BitConverter.ToString(r.GetData()).Replace('-', ' '));

        }

        private void FpgaHardReset()
        {
            Packet p = new Packet();
            Packet r;

            p.Type = Packet.eType.FPGAHARDRESET;
            p.Data = new byte[2];
            p.Data[0] = 0x01;//0x01;
            p.Data[1] = 0x00;

            Debug.WriteLine("Sent Hard Reset: " + BitConverter.ToString(p.GetData()).Replace('-', ' '));
            r = Execute(p);
            Debug.WriteLine("Received : " + BitConverter.ToString(r.GetData()).Replace('-', ' '));
        }

        private void FpgaInitWrite()
        {
            Packet p = new Packet();
            Packet r;

            p.Type = Packet.eType.INITWRITE;
            p.Data = new byte[2];
            p.Data[0] = p.Data[1] = 0xff;

            Debug.WriteLine("Sent InitWrite: " + BitConverter.ToString(p.GetData()).Replace('-', ' '));
            r = Execute(p);
            Debug.WriteLine("Received : " + BitConverter.ToString(r.GetData()).Replace('-', ' '));
        }

        private void trbAmp_Scroll(object sender, EventArgs e)
        {
            Packet p = new Packet();
            Packet r;

            p.Type = Packet.eType.SETDAC;
            p.Data = new byte[3];
            p.Data[0] = 0x01;   // first DAC is the Amp
            p.Data[1] = (byte)(trbAmp.Value / 256);
            p.Data[2] = (byte)(trbAmp.Value % 256);

            Debug.WriteLine("Sent InitWrite: " + BitConverter.ToString(p.GetData()).Replace('-', ' '));
            r = Execute(p);
            Debug.WriteLine("Received : " + BitConverter.ToString(r.GetData()).Replace('-', ' '));

        }

        private void rbSine_CheckedChanged(object sender, EventArgs e)
        {
            btnInitWrite_Click(sender, e);
            btnWriteRam_Click(sender, e);
            btnStartGen_Click(sender, e);
        }

        private void rbSaw_CheckedChanged(object sender, EventArgs e)
        {
            btnInitWrite_Click(sender, e);
            btnWriteRam_Click(sender, e);
            btnStartGen_Click(sender, e);
        }

        private void rbSquare_CheckedChanged(object sender, EventArgs e)
        {
            btnInitWrite_Click(sender, e);
            btnWriteRam_Click(sender, e);
            btnStartGen_Click(sender, e);
        }

        private void trbCoarse_Scroll(object sender, EventArgs e)
        {
            SetFreq();
        }

        private void SetFreq()
        {
            Packet p = new Packet();
            Packet r;

            // TODO: This uses STARTGEN now, should change to setfreq
            p.Type = Packet.eType.STARTGEN;
            p.Data = new byte[7];

            byte[] b1 = BitConverter.GetBytes(trbCoarse.Value);
            byte[] b2 = BitConverter.GetBytes(trbFine.Value);

            p.Data[0] = b2[0];
            p.Data[1] = b2[1];
            p.Data[2] = b1[0];
            p.Data[3] = b1[1];

            int setf = trbCoarse.Value * 65536 + trbFine.Value;
            double d = 25000000d / 0x0100000000 * setf;
            label5.Text = "Freq: " + d.ToString();

            p.Data[4] = 0x00;
            p.Data[5] = 0x00;

            p.Data[6] = 0x00;

            Debug.WriteLine("Sent Clock: " + BitConverter.ToString(p.GetData()).Replace('-', ' '));
            r = Execute(p);
            Debug.WriteLine("Received : " + BitConverter.ToString(r.GetData()).Replace('-', ' '));
        }

        private void trbFine_Scroll(object sender, EventArgs e)
        {
            SetFreq();
        }


    }
}
